"""This module contains the process for converting the World Bank's Documents and Rerports
metadata into the format that can be used for the LLM4Data.
"""
import json
import os
import re
from typing import Any, Dict, List, Optional, Tuple, Union
from urllib.parse import urlparse

import pandas as pd
from tqdm import tqdm


WHITESPACE_RE = re.compile("\s+")


class WBDocsSchema:

    def __init__(self, metadata) -> None:
        self.metadata = metadata
        self

# class WBDocsNormalizer:
#     def __init__(self, metadata):
#         self.metadata = metadata

#     def normalize(self):
#         metadata = self.metadata
#         item = {}

#         item["abstract"] = self.get_abstract(metadata)
#         item["adm_region"] = self.get_adm_region(metadata)
#         item["author"] = self.get_author(metadata)

#         item["collection"] = self.get_collection(metadata)
#         item["corpus"] = "WB"
#         item["country"] = self.get_country(metadata)

#         item["date_published"] = self.get_date_published(metadata)
#         item["digital_identifier"] = self.get_digital_identifier(metadata)
#         item["doc_type"] = self.get_doc_type(metadata)

#         item["geo_region"] = self.get_geo_region(metadata)

#         item["keywords"] = self.get_keywords(metadata)

#         item["language_src"] = self.get_language_src(metadata)

#         item["major_doc_type"] = self.get_major_doc_type(metadata)

#         item["project_id"] = self.get_wb_project_id(metadata)

#         item["title"] = self.get_title(metadata)
#         item["topics_src"] = self.get_topics_src(metadata)

#         item["url_pdf"] = metadata.get("pdfurl")
#         item["url_source"] = metadata.get("url_friendly_title", metadata.get("url"))
#         item["url_txt"] = metadata.get("txturl")

#         item["year"] = item["date_published"].split(
#             "-")[0] if (item["date_published"] and "-" in item["date_published"]) else None

#         item["src_metadata"] = metadata

#         return item

#     def normalize_set(self, s):
#         # s = set(['Publications & Research', 'Publications'])

#         l = sorted(s)
#         remove_index = set()

#         for i in range(len(l) - 1):
#             for j in range(1, len(l)):
#                 if l[i] in l[j]:
#                     remove_index.add(i)

#         for k in sorted(remove_index, reverse=True):
#             l.pop(k)

#         return l

#     def make_unique_entry(self, entries):
#         # This will remove duplicate entries in fields: `majdocty` (`majdoctype` : normalized) and `admreg`
#         entries = entries.strip()
#         if entries:
#             entries = self.normalize_set(
#                 set([i.strip() for i in entries.split(',') if i.strip()]))
#             if len(entries) == 0:
#                 entries = None
#         else:
#             entries = None

#         return entries

#     def get_abstract(self, metadata):
#         # "abstracts": {
#         #     "cdata!": "This press release announces the\n            World Bank has approved six loans totaling two hundred\n            twenty-seven million dollars to Congo, India, Sudan,\n            Tunisia, and Zaire."
#         # },

#         abstract = ""
#         if "abstracts" in metadata and isinstance(metadata["abstracts"], dict):
#             abstract = metadata["abstracts"].get("cdata!")
#         return abstract

#     def get_adm_region(self, metadata):
#         # "admreg": "Africa,Africa,East Asia and Pacific,East Asia and Pacific,Europe and Central Asia,Europe and Central Asia",
#         return self.make_unique_entry(metadata.get('admreg', ''))

#     def get_author(self, metadata, delimiter=";"):
#         # "authors": {
#         #     "0": {
#         #       "author": "Sjoberg,Fredrik Matias"
#         #     },
#         #     "1": {
#         #       "author": "Mellon,Jonathan"
#         #     },
#         #     "2": {
#         #       "author": "Peixoto,Tiago Carneiro"
#         #     },
#         #     "3": {
#         #       "author": "Hemker,Johannes Zacharias"
#         #     },
#         #     "4": {
#         #       "author": "Tsai,Lily Lee"
#         #     }
#         # },

#         authors = metadata.get("authors")

#         authors_value = None
#         if authors and isinstance(authors, dict):
#             authors_value = self.get_unique_list(
#                 [author["author"] for author in authors.values()])
#             authors_value = delimiter.join(authors_value)

#         return self.standardize_authors_list(authors_value, delimiter=delimiter)

#     def standardize_authors_list(self, authors, delimiter=";"):
#         # authors = "Runji,Justin;Jose Rizal;Bonifacio, Andres; Damaso, Maria Clara"
#         if not isinstance(authors, str):
#             return []
#         return [re.sub(r"(.*),\s*(.*)", r"\2 \1", i.strip()) for i in authors.split(delimiter)]

#     def get_collection(self, metadata):
#         return metadata.get("colti")

#     def get_country(self, metadata, delimiter=";"):
#         # "count": "Congo, Republic of,India,Sudan,Tunisia,Congo, Democratic Republic of",
#         p = re.compile("(\S),(\S)")
#         country = p.sub(f"\\1{delimiter}\\2", metadata.get("count", ""))
#         country = country.split(delimiter)
#         country = country if country else None

#         return country

#     def get_date_published(self, metadata):
#         # "docdt": "1981-09-30T00:00:00Z",
#         date = metadata.get("docdt", "T").split("T")[0]
#         if not date:
#             date = None
#         return date

#     def get_digital_identifier(self, metadata):
#         # "docm_id": "090224b086412c6c",
#         return metadata.get("docm_id", "")

#     def get_doc_type(self, metadata):
#         # "docty": "Agenda",
#         return self.make_unique_entry(metadata.get('docty', ''))

#     def get_geo_region(self, metadata):
#         # "geo_regions": {
#         #     "0": {
#         #       "geo_region": "South Eastern Europe and Balkans"
#         #     },
#         #     "1": {
#         #       "geo_region": "South Eastern Europe and Balkans"
#         #     },
#         # },

#         geo_regions = metadata.get("geo_regions")

#         geo_region = None
#         if geo_regions and isinstance(geo_regions, dict):
#             geo_region = self.get_unique_list(
#                 [geo_region["geo_region"] for geo_region in geo_regions.values()])

#         return geo_region

#     def get_keywords(self, metadata):
#         # "keywd": {
#         #     "0": {
#         #       "keywd": "plastic waste; catchment; waste handling\n            practices; terrestrial environment; illegal dumping;\n            frequency of rainfall events"
#         #     }
#         # }
#         keywds = metadata.get("keywd")

#         keywords = None
#         if keywds and isinstance(keywds, dict):
#             keywds = [kwd.strip() for keywd in keywds.values()
#                       for kwd in WHITESPACE_RE.sub(" ", keywd["keywd"]).split(";")]
#             keywords = self.get_unique_list(keywds)

#         return keywords

#     def get_language_src(self, metadata):
#         # "lang": "English",
#         return metadata.get("lang")

#     def get_major_doc_type(self, metadata):
#         # "majdocty": "Publications & Research",
#         return self.make_unique_entry(metadata.get('majdocty', ''))

#     def get_pdf_link(self, metadata):
#         pdfurl = metadata.get("pdfurl")

#         if not pdfurl:
#             # Attempt to recover pdfurl from txturl.
#             pdfurl = metadata.get("txturl")

#             if pdfurl:
#                 pdfurl = pdfurl.replace("/text/", "/pdf/")
#                 pdfurl = f"{os.path.splitext(pdfurl)[0]}.pdf"

#         return pdfurl

#     def get_title(self, metadata):
#         # "display_title": [
#         #     {
#         #     "display_title": "Issues and options in the design\n            of GEF-supported trust funds for biodiversity conservation"
#         #     }
#         # ],
#         dspt = metadata.get("display_title", None)
#         if dspt:
#             if isinstance(dspt, str):
#                 dspt = dspt
#             elif isinstance(dspt, list):
#                 dspt = dspt[0].get("display_title")
#             else:
#                 raise ValueError(f"display_title format unknown: {dspt}")
#         return dspt

#     def get_topics_src(self, metadata):
#         # "historic_topic": "Education,Environment,Energy,Rural Development,Water Resources",
#         return metadata.get("historic_topic", "").split(",")

#     def get_wb_lending_instrument(self, metadata):
#         # "lndinstr": {
#         #     "0": {
#         #       "lndinstr": "Technical Assistance Loan"
#         #     },
#         #     "1": {
#         #       "lndinstr": "Technical Assistance Loan"
#         #     },
#         #     "2": {
#         #       "lndinstr": "Technical Assistance Loan"
#         #     }
#         # },

#         lndinstrs = metadata.get("lndinstr")

#         lending_instrument = None
#         if lndinstrs and isinstance(lndinstrs, dict):
#             lending_instrument = self.get_unique_list(
#                 [lndinstr["lndinstr"] for lndinstr in lndinstrs.values()])

#         return lending_instrument

#     def get_wb_major_theme(self, metadata):
#         # "majtheme": "Financial and private sector development,Public sector governance",
#         return self.get_unique_list(metadata.get("majtheme", "").split(","))

#     def get_wb_product_line(self, metadata):
#         # "prdln": "IBRD/IDA",
#         return metadata.get("prdln")

#     def get_wb_project_id(self, metadata):
#         # "projectid": "P003677,P008088,P000544,P004440",
#         return self.get_unique_list(metadata.get("projectid", "").split(","))

#     def get_wb_sector(self, metadata):
#         # "sectr": {
#         #     "0": {
#         #       "sector": "Transportation"
#         #     },
#         #     "1": {
#         #       "sector": "Agriculture, Fishing and Forestry"
#         #     }
#         # },
#         sectr = metadata.get("sectr")

#         sectors = None
#         if sectr and isinstance(sectr, dict):
#             sectors = self.get_unique_list(
#                 [sector["sector"] for sector in sectr.values()])

#         return sectors

#     def get_wb_subtopic_src(self, metadata):
#         # "subtopic": "Public Sector Economics,Municipal Financial Management,Strategic Debt Management,Banks & Banking Reform,External Debt",
#         return self.get_unique_list(metadata.get("subtopic", "").split(","))

#     def get_wb_theme(self, metadata):
#         # "theme": "State-owned enterprise restructuring and privatization,Other financial and private sector development,International financial standards and systems,Other public sector governance",
#         return self.get_unique_list(metadata.get("theme", "").split(","))

#     def get_unique_list(self, values):
#         unique_list = sorted(set([i.strip() for i in values if i.strip()]))

#         if not unique_list or unique_list == [""]:
#             unique_list = None

#         return unique_list
